/*
 * SimpleCount: Simple path couting program with length bounds
 * by KUT_KMHT (Shuto Kato, Satoshi Matsumoto, Takashi Harada, Shogo Takeuchi)
 * Copyright (c) 2023 KUT_KMHT
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#include <iostream>
#include <sstream>
#include <vector>
#include <deque>
#include <algorithm>
#include <chrono>

using namespace std;

typedef unsigned int uintx;
typedef unsigned long long int ullint;
typedef uintx Node;
typedef deque<Node> NodeQueue;
typedef vector<Node> NodeVector;
typedef vector<NodeVector> EdgeMatrix;

namespace {
  const uintx INF = (1<<28);
  double sec_limit = 60000.0;

  std::chrono::system_clock::time_point start;
  double getElapsed() {
    return std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now()-start).count();
  }
  
  class GraphInfo {
  public:
    uintx M, N, T=0;
    uintx maxLength = INF;
    EdgeMatrix Edges;
    Node t1, t2;
    
    GraphInfo() {
      NodeVector nv;
      Edges.push_back(nv);
      string s;
      while (std::getline(cin, s)) {
        char prefix;
        istringstream iss(s);
        iss >> prefix;
        if (prefix == 'c') {
          continue;
        } else if (prefix == 'p') {
          string s2;
          iss >> s2 >> N >> M;
          for (uint i=0; i<N; i++) {
            NodeVector tv;
            Edges.push_back(tv);
          }
        } else if (prefix == 'l') {
          iss >> maxLength;
        } else if (prefix == 't') {
          Node x, y;
          iss >> x >> y;
	  t1 = x; t2 = y;
	  T = 2;
        } else if (prefix == 'e') {
          Node x, y;
          iss >> x >> y;
          Edges[x].push_back(y);
          Edges[y].push_back(x);
        }
      }
    }
    ~GraphInfo() {}
  };
  
  class Searcher {
    GraphInfo &G;
    uintx  maxLength = INF;
    ullint numPath = 0ull;
    NodeVector usedNodes;
    // for bfs
    NodeQueue nodeQueue;
    NodeVector Dist;    
    // for backtrace
    NodeVector Parents;
  public:
    Searcher(GraphInfo &G): G(G) {
      maxLength = G.maxLength;
      Dist.resize(G.N+1);
      Parents.resize(G.N+1, 0);
    }
    ~Searcher() {}
    ullint getPathCount() {return numPath;}
  private:

    uintx initBFS(Node& firstNode) {
      nodeQueue.clear(); 
      std::fill(Dist.begin(), Dist.end(), INF);
      firstNode = G.t1;
      Dist[G.t1] = 0;
      for (uintx ei=1; ei<usedNodes.size(); ei++) {
	const Node nv = usedNodes[ei];
	Dist[nv] = Dist[firstNode] + 1;
	firstNode = nv;
      }
      nodeQueue.push_back(firstNode);
      Parents[firstNode] = 0;
      return Dist[firstNode];
    }

    bool backtrace(Node s, Node t, NodeVector &ShortestPath) {
      Node n = t;
      while (n != s) {
        Node p = Parents[n];
        if (p == 0) return false;
        ShortestPath.push_back(n);
        n = p;
      }
      return true;
    }
    template<int TN>
    bool isTerminal(const Node &v) {
      if (TN == 2) return (v == G.t2);
      return (v > G.t1);
    }
    template<int TN>
    uintx BFS(Node &firstNode, NodeVector &ShortestPath) {
      if (initBFS(firstNode) == INF) return INF;
      
      while (!nodeQueue.empty()) {
        const auto &v = nodeQueue.front();
        if (Dist[v] >= maxLength) return INF;
        nodeQueue.pop_front();
        for (const auto &w : (G.Edges[v])) {
          if (Dist[w] < INF) continue;
          Dist[w] = Dist[v] + 1;
          Parents[w] = v;
          if (Dist[w] > maxLength) continue;
          if (isTerminal<TN>(w)) {
            if (Dist[w] > maxLength) continue;
            // found Path
            backtrace(firstNode,w, ShortestPath);
            std::reverse(ShortestPath.begin(), ShortestPath.end());
            return Dist[w];
          }
          nodeQueue.push_back(w);
        }
      }
      return INF;
    }
  public:
    template<int TN>
    void countPath(const int &depth) {
      if (depth == 0) {
	usedNodes.clear();
	usedNodes.push_back(G.t1);
      }
      if (getElapsed() > sec_limit*1000.0) return;
      NodeVector ShortestPath;
      
      Node prevN = G.t1;
      const uintx result = BFS<TN>(prevN, ShortestPath);
      if (result > maxLength) return;

      for (const auto &nextN : ShortestPath) {
        //erase Edge
        NodeVector &epp = G.Edges[prevN];
        NodeVector &epn = G.Edges[nextN];
        epn.erase(remove_if(epn.begin(), epn.end(), [&prevN](Node i)->bool{ return i==prevN; }), epn.end());
        epp.erase(remove_if(epp.begin(), epp.end(), [&nextN](Node i)->bool{ return i==nextN; }), epp.end());
        countPath<TN>(depth+1);
        //undo
        epp.push_back(nextN);
        epn.push_back(prevN);

	usedNodes.push_back(nextN);
	prevN = nextN;
      }
      if (TN != 2) countPath<TN>(depth+1);
      
      for (uintx si=0; si<ShortestPath.size(); si++) {
        usedNodes.pop_back();
      }
      numPath++;
    }
  };
}

int main() {
  start = std::chrono::system_clock::now();
  GraphInfo G;
  Searcher S(G);
  if (G.T == 2) {
    S.countPath<2>(0);
  } else {
    // all-pairs
    G.t1 = 0;
    for (Node s=1; s<=G.N; s++) {
      if (getElapsed() > sec_limit*1000.0) break;
      G.t1 = s;
      S.countPath<0>(0);
    }
  }
  const double elapsed = getElapsed();
  if (elapsed > sec_limit*1000.0) {
    cerr << "timeout" << endl;
  } else {
    cout << S.getPathCount() << endl;
  }
  
  return 0;
}
