/*
 * SimpleCount: Simple path couting program with length bounds
 * by KUT_KMHT (Shuto Kato, Satoshi Matsumoto, Takashi Harada, Shogo Takeuchi)
 * Copyright (c) 2023 KUT_KMHT
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
#include <iostream>
#include <fstream>
#include <sstream>
#include <deque>
#include <vector>

using namespace std;
int INF = (1 << 30);
typedef int Node;
typedef deque<Node> NodeQueue;
typedef vector<Node> NodeVector;
typedef vector<NodeVector> EdgeMatrix;

class GraphInfo {
public:
  int M, N, T=0;
  int maxLength = 0;
  EdgeMatrix Edges;
  Node t1, t2;
  vector<int> sdist, tdist;
  GraphInfo() {
    {
      NodeVector nv;
      Edges.push_back(nv);
    }
    string s;
    while (std::getline(cin, s)) {
      char prefix;
      istringstream iss(s);
      iss >> prefix;
      if (prefix == 'c') continue;
      if (prefix == 'p') {
        string s2;
        iss >> s2 >> N >> M;
        for (int i=0; i<N; i++) {
          NodeVector nv;
          Edges.push_back(nv);
        }
      } else if (prefix == 'l') {
        iss >> maxLength;
      } else if (prefix == 't') {
        Node x, y;
        iss >> x >> y;
        t1 = x; t2 = y; T = 2;
      } else if (prefix == 'e') {
        Node x, y;
        iss >> x >> y;
        Edges[x].push_back(y);
        Edges[y].push_back(x);
      }
    }
  }
  ~GraphInfo() {}

  void printHeader(FILE *output, const int length) {
    ostringstream oss;
    for (int j=1; j<=length; j++) oss << "s" << j << " ";
    oss << "| ";
    for (int j=1; j<=N; j++)      oss << "v" << j << " ";
    for (int j=0; j<=length; j++) oss << "p" << j << " ";
    oss << "\n";
    fprintf(output, "%s", oss.str().c_str());
  }
  
  void internalBFS(vector<int> &dist, int first) {
    NodeQueue nodeQ;
    dist.resize(N+1, INF);
    dist[first] = 0;
    nodeQ.push_back(first);
    while (!nodeQ.empty()) {
      const auto v = nodeQ.front();
      nodeQ.pop_front();
      for (const auto &w : (Edges[v])) {
        if (dist[w] < INF) continue;
        dist[w] = dist[v] + 1;
        if (dist[w] > maxLength) continue;
        nodeQ.push_back(w);
      }
    }
  }
  void bfs() {
    if (T != 2) return;
    internalBFS(sdist, t1);
    internalBFS(tdist, t2);
  }

  void printOption(FILE *output, const int &s, const Node &v, const Node &w) {
    fprintf(output,"s%d v%d:%d v%d:%d p%d:%d p%d:%d\n",
	    s,  v, s-1, w, s,  s-1, v, s, w);
  }

  template <int TN>
  bool generateInput(const int &length) {
    string fn = "xcc_input.txt";
    FILE *output=fopen(fn.c_str(), "w");

    // header
    printHeader(output, length);

    if (TN == 2) {
      bfs();
      // l == 1
      if (length == 1) {
	bool includeOption = false;
	  for (const auto &v : Edges[t1]) {
	  if (v != t2) continue;
	  printOption(output, 1, t1, t2);
	  includeOption = true;
	}
	fclose(output);
	return includeOption;
      }
    
      // l  > 1
      // for terminal1
      for (const auto &v : Edges[t1]) {
        printOption(output, 1, t1, v);
      }

      // for all edges except for terminals
      for (int s=2; s<length; s++) {
        for (Node n=1; n<=N; n++) {
          if (n == t1 || n == t2) continue;
          for (const auto &v : Edges[n]) {
	    if (v == t1 || v == t2) continue;
            const int sd = std::min(sdist[n], sdist[v]);
            const int td = length - std::min(tdist[n], tdist[v]);
            if (sd > s || s > td) continue; // continue if not (sd <= s <= td)
            printOption(output, s, n, v);
          }
        }
      }
      
      // for terminal2
      for (const auto &v : Edges[t2]) {
        printOption(output, length, v, t2);
      }
    } else {
      // all-pairs
      for (int s=1; s<=length; s++) {
        for (Node n=1; n<=N; n++) {
          for (const auto &v : Edges[n]) {
            printOption(output, s, n, v);
          }
        }
      }
    }
    fclose(output);
    return true;
  }
};

int main() {
  vector<string> files;
  GraphInfo G;

  {
    ostringstream oss;
    oss << "rm -f xcc_result.txt\n";
    system(oss.str().c_str());
  }
  
  for (int i=1; i<=G.maxLength; i++) {
    ostringstream oss;
    oss << "./cover/bin/xcc -v -1 ";
    oss << " xcc_input.txt >> xcc_result.txt 2>&1\n";
    
    if (G.T == 2) { // two-terminal
      if (! G.generateInput<2>(i)) continue;
    } else { // all pair
      G.generateInput<0>(i);
    }
    system(oss.str().c_str());
  }

  ostringstream oss;
  oss << "grep solution xcc_result.txt|awk \'{sum+=$4} END {print sum";
  if (G.T != 2) oss << "/2";
  oss << "}\'";
  system(oss.str().c_str());
  
  // cleaning files
  oss.str("");
  oss.clear();
  oss << "rm -f xcc_input.txt xcc_result.txt \n";
  system(oss.str().c_str());
  return 0;
}
